from . import utils

class EventContainer:
    def __init__(self):
        self.commands = {}
        self.components = {}
        self.user_commands = {}
        self.message_commands = {}
        self.events = {}

    def command(self, command):
        if command.type == utils.SLASH:
            self.commands[command.name] = command.func
        elif command.type == utils.USER:
            self.user_commands[command.name] = command.func
        elif command.type == utils.MESSAGE:
            self.message_commands[command.name] = command.func

    def component(self, component):
        self.components[component.ucid] = component.func

    def event(self, event):
        try:
            self.events[event.name].append(event.func)
        except KeyError:
            self.events[event.name] = [event.func]


import inspect


class Command():
    def __init__(self, name, func, _type):
        self.name = name
        self.func = func
        self.type = _type

def command(name, _type):
    def predicate(func):
        if not inspect.iscoroutinefunction(func):
            raise TypeError("Command functions must be coroutine.")
        return Command(name, func, _type)
    return predicate